/* vbm_structures.h
 *
 * Command structure definitions
 *
 * This file can be used by Lua FFI.
 *
 */

#pragma pack(push)
#pragma pack(1)

typedef enum {
    VBM_TS_OFFSET = 1, VBM_GET_TS,
    VBM_SETUP_SYNC_TS, VBM_GET_SYNC_TS,
    /* [POWER] transducer power */
    VBM_TXDR_POWER_ON, VBM_TXDR_POWER_OFF,
    /* [CABLE] cable sense */
    VBM_GET_CABLE_SENSE,
    /* [EVENT] event processing */
    VBM_EVENT_SETUP,
    VBM_EVENT_START, VBM_EVENT_STOP,
    /* [SYNC] sync clock for eDAQ */
    VBM_SETUP_SYNC,
    /* [VBM] serial */
    VBM_SETUP_VBM,
    VBM_RX_START, VBM_RX_STOP,
    VBM_TX_START, VBM_TX_STOP,
    /* [ECM] hybrid/custom */
    VBM_SETUP_ECM,
    VBM_LAST_COMMAND = 0x7FFFFFFF    /* Insures 32-bit enum */
} vbm_cmd_t;

typedef struct {
    cmd_rate_parameters_s       baud_rate_lo_speed;
    cmd_rate_parameters_s       baud_rate_hi_speed;
    cmd_rate_parameters_s       baud_rate_full_speed;
    cmd_rate_parameters_s       pwm_monitor;
    uint32_t                    packet_timeout_ts_count;
} vbm_cmd_vbm_s;

typedef struct {
    cmd_rate_parameters_s       baud_rate;
    cmd_rate_parameters_s       update_rate;
    int64_t                     start_time;
    int64_t                     initial_value;
    int64_t                     increment_value;
} vbm_cmd_sync_s;

typedef struct {
    vbm_cmd_vbm_s               vbm;
    vbm_cmd_sync_s              sync;
} vbm_cmd_ecm_s;

typedef struct {
    uint32_t                    valid_bytes;
    uint32_t                    static_data[6];
} vbm_cmd_update_s;

typedef struct {
    int64_t                     start_time;
    cmd_rate_parameters_s       flush_interval;
    cmd_rate_parameters_s       vbm_clk_rate;
    uint32_t                    phys_address;
} vbm_cmd_rx_start_s;

typedef struct {
    uint32_t                    phys_address;
    uint32_t                    channel_id;
} can_cmd_tx_start_stop_s;

typedef struct {
    uint32_t                    msg_length;
    uint8_t                     msg[64];
} vbm_cmd_tx_msg_s;

typedef struct {
    uint8_t                     cable_unplugged;
    uint8_t                     cable_plugged;
    uint8_t                     vbm_plugged;
    uint8_t                     vbm_unplugged;
} vbm_cmd_event_vector_s;

typedef struct {
    vbm_cmd_t                   cmd;
    uint32_t                    cmd_id;
    union {
        cmd_ts_offset_s         ts_offset;
        vbm_cmd_vbm_s           vbm;
        vbm_cmd_sync_s          sync;
        vbm_cmd_ecm_s           ecm;
        vbm_cmd_update_s        update;
        vbm_cmd_rx_start_s      rx_start;
        can_cmd_tx_start_stop_s tx_start_stop;
        vbm_cmd_tx_msg_s        tx_msg;
        cmd_event_start_s       ev_start;
        vbm_cmd_event_vector_s  events;
    } prm;
} vbm_cmd_s;

typedef struct {
    int64_t                     ts;
    uint8_t                     is_control_frame;
    uint8_t                     msg_length;
    uint8_t                     reserved[2];
    uint8_t                     msg[64];
} vbm_rxtx_buffer_t;

typedef struct {
    int64_t                     ts;
    vbm_cmd_event_vector_s      events;
} vbm_ev_buffer_t;

enum { MAX_VBM_TX_STARTS = 16 };

#pragma pack(pop)
