#!/usr/bin/lua

require('lj_include'){
    'sys/types',
    'unistd',
    'sched',
    'linux/sched',
}

local log = require 'log'
-- Open logging session for the entire process
-- NOTE: Lua modules, loaded via 'require', should *NOT* call "log:open()"
log:open('flagship-proxy', log.USER)

local ffi = require 'ffi'
local ev = require 'ev'
local u = require 'unix'
local cc = require 'connctrl'
local config = require 'system_config'
local armada = require 'armada'

local C = ffi.C

local m = require 'register_critical'
m.RegisterMe()

-- Increase process priority
--C.nice(-5);

local sp = ffi.new('struct sched_param')
sp.sched_priority = 50 -- Arbitrary for now
assert(C.sched_setscheduler(C.getpid(), C.SCHED_RR, sp) == 0)

local function dbg(fmt, ...)
    log:log(log.DEBUG, fmt, ...)
end

local proxies = {}
local proxy_map

if not u.stat('/sys/class/cc_class/cca3') then
    error("Driver not loaded or FPGA unconfigured")
end

if config.device_type == 'cx23r' then
    proxy_map = {
        {'can', 'can1', '/dev/cca0'},
        {'can', 'can2', '/dev/cca1'},
        {'can', 'can3', '/dev/cca2'},
        {'aux', 'aux',  '/dev/cca3'},
        {'gps', 'gps',  '/dev/cca4'},
        {'dio', 'dio',  '/dev/cca5'},
    }
else
    proxy_map = {
        {'dio', 'dio',  '/dev/cca5'},
    }
end

if config.device_type == 'cx23r' then
    proxies.can = require 'can_proxy'
    proxies.gps = require 'gps_proxy'
    proxies.aux = require 'aux_proxy'
end
proxies.dio = require 'dio_proxy'

for _,v in ipairs(proxy_map) do cc.controller_reset(v[3]) end

local ps = {}

for i, v in ipairs(proxy_map) do
    if proxies[v[1]] then
        ps[#ps + 1] = proxies[v[1]]:new(v[2], v[3])
    end
end

ev.Signal.new(function(loop) loop:unloop() end, u.SIGINT):start(ev.Loop.default)
ev.Signal.new(function(loop) loop:unloop() end, u.SIGTERM):start(ev.Loop.default)

local ok, err = xpcall(function() ev.Loop.default:loop() end, debug.traceback)

for _, p in ipairs(ps) do
    dbg('halt', p)
    p:halt()
end
if not ok then
    error(err .. "\n")
end
