local ffi = require 'ffi'
local lj_include = require 'lj_include'

lj_include('bits_types_struct_iovec')

ffi.cdef[[
enum __pid_type {
    F_OWNER_TID = 0,
    F_OWNER_PID,
    F_OWNER_PGRP,
    F_OWNER_GID = F_OWNER_PGRP
};
struct f_owner_ex {
    enum __pid_type type;
    __pid_t pid;
};
]]

lj_include('linux_falloc')

ffi.cdef[[
struct file_handle {
    unsigned int handle_bytes;
    int handle_type;
    unsigned char f_handle[0];
};
__ssize_t readahead(int __fd, __off64_t __offset, size_t __count);
int sync_file_range(int __fd, __off64_t __offset, __off64_t __count,
    unsigned int __flags);
__ssize_t vmsplice(int __fdout, const struct iovec *__iov,
    size_t __count, unsigned int __flags);
__ssize_t splice(int __fdin, __off64_t * __offin, int __fdout,
    __off64_t * __offout, size_t __len, unsigned int __flags);
__ssize_t tee(int __fdin, int __fdout, size_t __len, unsigned int __flags);
int fallocate(int __fd, int __mode, __off_t __offset, __off_t __len);
int fallocate64(int __fd, int __mode, __off64_t __offset, __off64_t __len);
int name_to_handle_at(int __dfd, const char *__name,
    struct file_handle *__handle, int *__mnt_id, int __flags);
int open_by_handle_at(int __mountdirfd, struct file_handle *__handle,
    int __flags);
]]

local defs = {
    O_ACCMODE = '0003',
    O_RDONLY = '00',
    O_WRONLY = '01',
    O_RDWR = '02',
    O_CREAT = '0100',
    O_EXCL = '0200',
    O_NOCTTY = '0400',
    O_TRUNC = '01000',
    O_APPEND = '02000',
    O_NONBLOCK = '04000',
    O_NDELAY = '04000',
    O_SYNC = '04010000',
    O_FSYNC = '04010000',
    O_ASYNC = '020000',
    F_GETLK = '5',
    F_SETLK = '6',
    F_SETLKW = '7',
    F_GETLK64 = '12',
    F_SETLK64 = '13',
    F_SETLKW64 = '14',
    F_OFD_GETLK = '36',
    F_OFD_SETLK = '37',
    F_OFD_SETLKW = '38',
    F_DUPFD = '0',
    F_GETFD = '1',
    F_SETFD = '2',
    F_GETFL = '3',
    F_SETFL = '4',
    F_SETLEASE = '1024',
    F_GETLEASE = '1025',
    F_NOTIFY = '1026',
    F_SETPIPE_SZ = '1031',
    F_GETPIPE_SZ = '1032',
    F_ADD_SEALS = '1033',
    F_GET_SEALS = '1034',
    F_GET_RW_HINT = '1035',
    F_SET_RW_HINT = '1036',
    F_GET_FILE_RW_HINT = '1037',
    F_SET_FILE_RW_HINT = '1038',
    F_DUPFD_CLOEXEC = '1030',
    FD_CLOEXEC = '1',
    F_RDLCK = '0',
    F_WRLCK = '1',
    F_UNLCK = '2',
    F_EXLCK = '4',
    F_SHLCK = '8',
    LOCK_SH = '1',
    LOCK_EX = '2',
    LOCK_NB = '4',
    LOCK_UN = '8',
    LOCK_MAND = '32',
    LOCK_READ = '64',
    LOCK_WRITE = '128',
    LOCK_RW = '192',
    DN_ACCESS = '0x00000001',
    DN_MODIFY = '0x00000002',
    DN_CREATE = '0x00000004',
    DN_DELETE = '0x00000008',
    DN_RENAME = '0x00000010',
    DN_ATTRIB = '0x00000020',
    DN_MULTISHOT = '0x80000000',
    F_SEAL_SEAL = '0x0001',
    F_SEAL_SHRINK = '0x0002',
    F_SEAL_GROW = '0x0004',
    F_SEAL_WRITE = '0x0008',
    RWF_WRITE_LIFE_NOT_SET = '0',
    RWH_WRITE_LIFE_NONE = '1',
    RWH_WRITE_LIFE_SHORT = '2',
    RWH_WRITE_LIFE_MEDIUM = '3',
    RWH_WRITE_LIFE_LONG = '4',
    RWH_WRITE_LIFE_EXTREME = '5',
    FAPPEND = '02000',
    FFSYNC = '04010000',
    FASYNC = '020000',
    FNONBLOCK = '04000',
    FNDELAY = '04000',
    POSIX_FADV_NORMAL = '0',
    POSIX_FADV_RANDOM = '1',
    POSIX_FADV_SEQUENTIAL = '2',
    POSIX_FADV_WILLNEED = '3',
    SYNC_FILE_RANGE_WAIT_BEFORE = '1',
    SYNC_FILE_RANGE_WRITE = '2',
    SYNC_FILE_RANGE_WAIT_AFTER = '4',
    SPLICE_F_MOVE = '1',
    SPLICE_F_NONBLOCK = '2',
    SPLICE_F_MORE = '4',
    SPLICE_F_GIFT = '8',
    MAX_HANDLE_SZ = '128',
}
local format, cdef, C = string.format, ffi.cdef, ffi.C
for sym, val in pairs(defs) do
    if not pcall(function() return C[sym] end) then
        cdef(format("enum {%s = %s};", sym, val))
    end
end

