local ffi = require 'ffi'
local lj_include = require 'lj_include'

lj_include('stddef')
lj_include('sys_types')

ffi.cdef[[
typedef __socklen_t socklen_t;
]]

lj_include('bits_socket_type')
lj_include('bits_sockaddr')

ffi.cdef[[
struct sockaddr {
    sa_family_t sa_family;
    char sa_data[14];
};
struct sockaddr_storage {
    sa_family_t ss_family;
    char __ss_padding[(128 - (sizeof(unsigned short int)) -
	    sizeof(unsigned long int))];
    unsigned long int __ss_align;
};
enum {
    MSG_OOB = 0x01,
    MSG_PEEK = 0x02,
    MSG_DONTROUTE = 0x04,
    MSG_TRYHARD = MSG_DONTROUTE,
    MSG_CTRUNC = 0x08,
    MSG_PROXY = 0x10,
    MSG_TRUNC = 0x20,
    MSG_DONTWAIT = 0x40,
    MSG_EOR = 0x80,
    MSG_WAITALL = 0x100,
    MSG_FIN = 0x200,
    MSG_SYN = 0x400,
    MSG_CONFIRM = 0x800,
    MSG_RST = 0x1000,
    MSG_ERRQUEUE = 0x2000,
    MSG_NOSIGNAL = 0x4000,
    MSG_MORE = 0x8000,
    MSG_WAITFORONE = 0x10000,
    MSG_BATCH = 0x40000,
    MSG_ZEROCOPY = 0x4000000,
    MSG_FASTOPEN = 0x20000000,
    MSG_CMSG_CLOEXEC = 0x40000000
};
struct msghdr {
    void *msg_name;
    socklen_t msg_namelen;
    struct iovec *msg_iov;
    size_t msg_iovlen;
    void *msg_control;
    size_t msg_controllen;
    int msg_flags;
};
struct cmsghdr {
    size_t cmsg_len;
    int cmsg_level;
    int cmsg_type;
    unsigned char __cmsg_data[];
};
struct cmsghdr *__cmsg_nxthdr(struct msghdr *__mhdr,
    struct cmsghdr *__cmsg);
enum {
    SCM_RIGHTS = 0x01, SCM_CREDENTIALS = 0x02
};
struct ucred {
    pid_t pid;
    uid_t uid;
    gid_t gid;
};
]]

lj_include('asm_socket')

ffi.cdef[[
struct linger {
    int l_onoff;
    int l_linger;
};
]]

local defs = {
    PF_UNSPEC = '0',
    PF_LOCAL = '1',
    PF_UNIX = '1',
    PF_FILE = '1',
    PF_INET = '2',
    PF_AX25 = '3',
    PF_IPX = '4',
    PF_APPLETALK = '5',
    PF_NETROM = '6',
    PF_BRIDGE = '7',
    PF_ATMPVC = '8',
    PF_X25 = '9',
    PF_INET6 = '10',
    PF_ROSE = '11',
    PF_DECnet = '12',
    PF_NETBEUI = '13',
    PF_SECURITY = '14',
    PF_KEY = '15',
    PF_NETLINK = '16',
    PF_ROUTE = '16',
    PF_PACKET = '17',
    PF_ASH = '18',
    PF_ECONET = '19',
    PF_ATMSVC = '20',
    PF_RDS = '21',
    PF_SNA = '22',
    PF_IRDA = '23',
    PF_PPPOX = '24',
    PF_WANPIPE = '25',
    PF_LLC = '26',
    PF_IB = '27',
    PF_MPLS = '28',
    PF_CAN = '29',
    PF_TIPC = '30',
    PF_BLUETOOTH = '31',
    PF_IUCV = '32',
    PF_RXRPC = '33',
    PF_ISDN = '34',
    PF_PHONET = '35',
    PF_IEEE802154 = '36',
    PF_CAIF = '37',
    PF_ALG = '38',
    PF_NFC = '39',
    PF_VSOCK = '40',
    PF_KCM = '41',
    PF_QIPCRTR = '42',
    PF_SMC = '43',
    PF_MAX = '44',
    AF_UNSPEC = '0',
    AF_LOCAL = '1',
    AF_UNIX = '1',
    AF_FILE = '1',
    AF_INET = '2',
    AF_AX25 = '3',
    AF_IPX = '4',
    AF_APPLETALK = '5',
    AF_NETROM = '6',
    AF_BRIDGE = '7',
    AF_ATMPVC = '8',
    AF_X25 = '9',
    AF_INET6 = '10',
    AF_ROSE = '11',
    AF_DECnet = '12',
    AF_NETBEUI = '13',
    AF_SECURITY = '14',
    AF_KEY = '15',
    AF_NETLINK = '16',
    AF_ROUTE = '16',
    AF_PACKET = '17',
    AF_ASH = '18',
    AF_ECONET = '19',
    AF_ATMSVC = '20',
    AF_RDS = '21',
    AF_SNA = '22',
    AF_IRDA = '23',
    AF_PPPOX = '24',
    AF_WANPIPE = '25',
    AF_LLC = '26',
    AF_IB = '27',
    AF_MPLS = '28',
    AF_CAN = '29',
    AF_TIPC = '30',
    AF_BLUETOOTH = '31',
    AF_IUCV = '32',
    AF_RXRPC = '33',
    AF_ISDN = '34',
    AF_PHONET = '35',
    AF_IEEE802154 = '36',
    AF_CAIF = '37',
    AF_ALG = '38',
    AF_NFC = '39',
    AF_VSOCK = '40',
    AF_KCM = '41',
    AF_QIPCRTR = '42',
    AF_SMC = '43',
    AF_MAX = '44',
    SOL_RAW = '255',
    SOL_DECNET = '261',
    SOL_X25 = '262',
    SOL_PACKET = '263',
    SOL_ATM = '264',
    SOL_AAL = '265',
    SOL_IRDA = '266',
    SOL_NETBEUI = '267',
    SOL_LLC = '268',
    SOL_DCCP = '269',
    SOL_NETLINK = '270',
    SOL_TIPC = '271',
    SOL_RXRPC = '272',
    SOL_PPPOL2TP = '273',
    SOL_BLUETOOTH = '274',
    SOL_PNPIPE = '275',
    SOL_RDS = '276',
    SOL_IUCV = '277',
    SOL_CAIF = '278',
    SOL_ALG = '279',
    SOL_NFC = '280',
    SOL_KCM = '281',
    SOL_TLS = '282',
    SOMAXCONN = '128',
}
local format, cdef, C = string.format, ffi.cdef, ffi.C
for sym, val in pairs(defs) do
    if not pcall(function() return C[sym] end) then
        cdef(format("enum {%s = %s};", sym, val))
    end
end

