-- mx_support
local log = require 'log'

local m = {}

function m.apply_quarter_bridge_offset(spec)
    if not spec.connector:find("mx1615") then return end
    if spec.sensor_input and spec.sensor_input.mode and
        spec.sensor_input.type and spec.sensor_input.wires and
        spec.sensor_input.regulation ~= nil and
        spec.sensor_input.mode == 'bridge' and
        spec.sensor_input.type == 'quarter' and
        tonumber(spec.sensor_input.wires) == 3 and
        spec.sensor_input.regulation == false then

        local offset_table = {
            pre3_2 = {
                ["350"] = {
                    -1.938145,
                    -2.004457,
                    -1.963448,
                    -1.998397,
                    -1.968783,
                    -2.053180,
                    -1.981609,
                    -2.075157,
                    -1.965713,
                    -2.031774,
                    -1.947282,
                    -1.996316,
                    -2.026295,
                    -2.067507,
                    -2.082926,
                    -2.029367,
                },
                ["120"] = {
                    -5.596721,
                    -5.808038,
                    -5.696278,
                    -5.773309,
                    -5.701307,
                    -5.970618,
                    -5.718924,
                    -6.027020,
                    -5.696303,
                    -5.874288,
                    -5.619076,
                    -5.801032,
                    -5.875294,
                    -6.000510,
                    -6.027146,
                    -5.841765,
                }
            },
            post3_2 = {
                ["350"] = {
                    -1.9196,
                    -1.9877,
                    -1.9444,
                    -1.9747,
                    -1.9582,
                    -2.0398,
                    -1.9498,
                    -2.0376,
                    -1.9642,
                    -2.0119,
                    -1.9827,
                    -1.9827,
                    -2.0065,
                    -2.0426,
                    -2.0548,
                    -2.0081,
                },
                ["120"] = {
                    -5.5473,
                    -5.7733,
                    -5.6540,
                    -5.7104,
                    -5.6786,
                    -5.9355,
                    -5.6364,
                    -5.9280,
                    -5.6917,
                    -5.8160,
                    -5.5740,
                    -5.7621,
                    -5.8175,
                    -5.9381,
                    -5.9554,
                    -5.7911,
                }
            },
        }

        local function get_slope_e2o()
            local slope_e2p
            for _, c in ipairs(spec.calibration) do
                if c.purpose and c.purpose == 'lead_wire_correction' then
                    slope_e2p = tonumber(c.slope)
                    break
                end
            end
            if not slope_e2p then
                for _, c in ipairs(spec.calibration) do
                    if c.purpose and c.purpose == 'sensor_scaling' then
                        if c.mode == 'defined_slope_intercept' then
                            slope_e2p = tonumber(c.slope)
                        elseif c.mode == 'defined_sensitivity_factor' then
                            slope_e2p = tonumber(c.sensitivity)
                        elseif c.mode == 'defined_zero_span' or
                            c.mode == 'shunt' then
                            slope_e2p = tonumber(c.physical_span) /
                                tonumber(c.electrical_span)
                        elseif c.mode == 'defined_two_point' or
                            c.mode == 'experimental_two_point' then
                            slope_e2p =
                                (tonumber(c.physical[2]) -
                                    tonumber(c.physical[1])) /
                                (tonumber(c.electrical[2]) -
                                    tonumber(c.electrical[1]))
                        elseif c.mode == 'strain_gage' then
                            slope_e2p = 4000.0 /
                                (tonumber(c.gage_factor) *
                                tonumber(c.bridge_factor))
                        end
                        break
                    end
                end
            end
            if slope_e2p then
                local slope_p2o = 1

                for _, c in ipairs(spec.calibration) do
                    if c.purpose and c.purpose == 'sensor_scaling' then
                        if c.mode == 'strain_gage' then
                            for _, c in ipairs(spec.calibration) do
                                if c.purpose and
                                    c.purpose == 'units_conversion' then
                                    local p2o_table = {
                                        ["µm/m"] = 1,
                                        ["µe"] = 1,
                                        ["µstrain"] = 1,
                                        ["microstrain"] = 1,
                                        ["mm/m"] = 1.0e-03,
                                        ["m/m"] = 1.0e-06,
                                        ["strain"] = 1.0e-06,
                                    }
                                    slope_p2o =
                                        p2o_table[spec.output_units:lower()]
                                    if not slope_p2o then
                                        error(
                                            'Unable to compensate 1/4 ' ..
                                            'bridge - unknown unit type: ' ..
                                            spec.output_units)
                                    end
                                    break
                                end
                            end
                        end
                        break
                    end
                end
                return slope_e2p * slope_p2o
            end
            error('Unable to compensate 1/4 bridge - e2p unknown')
        end

        local slope_e2o = get_slope_e2o()
        if type(slope_e2o) == 'string' then return slope_e2o end
        local cnum = spec.connector:match('onnector(%d+)%.mx')
        if not cnum then
            error('Unable to compensate 1/4 bridge - unknown channel number')
        end
        cnum = tonumber(cnum)

        if not spec.mx1615br_analog_hardware_revision then
            error('Channel hardware version information not available')
        end
        if not tonumber(spec.mx1615br_analog_hardware_revision) then
            error(string.format(
                'Can not find channel HW revision for channel "%s"', spec.name))
        end
        local ot = offset_table.pre3_2
        if tonumber(spec.mx1615br_analog_hardware_revision) >= 3.2 then
            ot = offset_table.post3_2
            log:log(log.NOTICE, string.format(
                "Using 3.20 offset table on channel %s", spec.name))
        else
            log:log(log.NOTICE, string.format(
                "Using 3.01 offset table on channel %s", spec.name))
        end
        if not ot[tostring(spec.bridge_resistance)] then
            error('Unable to compensate 1/4 bridge - unknown bridge resistance')
        end

        local unscaled_hw_offset = ot[tostring(spec.bridge_resistance)][cnum]
        if not unscaled_hw_offset then
            error('Unable to compensate 1/4 bridge - unscaled offset unknown')
        end

        local scaled_hw_offset = unscaled_hw_offset * slope_e2o
        for _, c in ipairs(spec.calibration) do
            if c.purpose and c.purpose == 'rezero' then
                c.intercept = tonumber(c.intercept) - scaled_hw_offset
                return
            end
        end
        error('Unable to compensate 1/4 bridge - no rezero calibration step')
    end
end

return m
