local ffi = require'ptpv2-ffi/cdata'
local C = ffi.C
require'ptpv2-ffi/internal-time'

local m = {}

function m:send()
    local ci = self.clock_info
    local size = ci.sizeof_delay_req_msg

    if ci.waiting_delay_req_tx_ts > 0 then
        self:log("warning",
            "send dreq while waiting on previous dreq tx timestamp")
        return
    end
    
    ffi.fill(ci.ibuf, size + 2)
    self:build_msg_header()
    local msg = ffi.cast('delay_req_msg_t *', ci.ibuf)
    msg.hdr.transport_specific_message_type = C.PTP_MSG_TYPE_DELAY_REQ
    msg.hdr.message_length = C.htons(size)
    msg.hdr.sequence_id = C.htons(ci.sent_delay_req_sequence_id)
    msg.hdr.control_field = 0x1
    msg.hdr.log_message_interval = 0x7f

    -- This really isn't used with two step clocks
    local internal_time = self.timing_iface:get_time()
    local origin_timestamp = internal_time:to_timestamp()
    msg.origin_timestamp.seconds.msb =
        C.htons(origin_timestamp.seconds.msb)
    msg.origin_timestamp.seconds.lsb =
        C.htonl(origin_timestamp.seconds.lsb)
    msg.origin_timestamp.nanoseconds =
        C.htonl(origin_timestamp.nanoseconds)

    if ci.transport_is_ipv6 ~= 0 then
        size = size + 2
    end

    local sock = self[C.EVENT_PORT]
    ci.waiting_delay_req_tx_ts = 1
    local r, e = pcall(sock.sendto, sock, ci.ibuf, size, function(ts, raw)
            self.delay_req_send_time = InternalTime:new(
                self.timing_iface:raw_ts_to_time(raw))
            ci.waiting_delay_req_tx_ts = 0
        end)
    if not r and not e:find('No message') then
        self:log("error",
            string.format("send_delay_req(): sendto() failed: %s", e))
        self:to_state(C.PTP_STATE_FAULTY)
        return
    elseif r then
        ci.sent_delay_req_sequence_id = ci.sent_delay_req_sequence_id + 1
    end
end

function m:faulty(time, from_me)
    -- FIXME: Try to recover from faulty state?
end

function m:slave(time, from_me)
    if from_me then
        self.delay_request_send_time = time + self.outbound_latency
    end
end

function m:master(time, from_me)
    local f = self.send[C.PTP_MSG_TYPE_DELAY_RESP]
    f(self, time)
end

m.handle_state = {}
m.handle_state[C.PTP_STATE_FAULTY] = m.faulty
m.handle_state[C.PTP_STATE_SLAVE] = m.slave
m.handle_state[C.PTP_STATE_MASTER] = m.master

function m:handle(time, from_me)
    local state = self.clock_info.state
    local f = m.handle_state[state]
    if f then
        f(self, time, from_me)
    end
end

return m
