local ffi = require'ptpv2-ffi/cdata'
local C = ffi.C

local m = {}

function m:tx_ts_cb(rs, raw)
    local time = InternalTime:new(self.timing_iface:raw_ts_to_time(raw))
    self.send[C.PTP_MSG_TYPE_FOLLOW_UP](self, time + self.outbound_latency)
    self.sync_interval_timer:start(ev.Loop.default)
end

function m:send()
    local ci = self.clock_info
    local size = ci.sizeof_sync_msg
    
    ffi.fill(ci.ibuf, size + 2)
    self:build_msg_header()
    local msg = ffi.cast('sync_msg_t *', ci.ibuf)
    msg.hdr.transport_specific_message_type = C.PTP_MSG_TYPE_SYNC
    msg.hdr.message_length = C.htons(size)
    msg.hdr.sequence_id = C.htons(ci.sent_sync_sequence_id)
    msg.hdr.log_message_interval = ci.log_sync_interval

    -- This really isn't used with two step clocks
    local internal_time = self.timing_iface:get_time()
    local origin_timestamp = internal_time:to_timestamp()
    msg.origin_timestamp.seconds.msb =
        C.htons(origin_timestamp.seconds.msb)
    msg.origin_timestamp.seconds.lsb =
        C.htonl(origin_timestamp.seconds.lsb)
    msg.origin_timestamp.nanoseconds =
        C.htonl(origin_timestamp.nanoseconds)

    if ci.transport_is_ipv6 ~= 0 then
        size = size + 2
    end

    local sock = self[C.EVENT_PORT]

    local r, e = pcall(sock.sendto, sock, ci.ibuf, size, function(ts, raw)
            m.tx_ts_cb(self, ts, raw)
        end)
    if not r and not e:find('No message') then
        self:log("error", string.format("send_sync(): sendto() failed - %s", e))
        self:to_state(C.PTP_STATE_FAULTY)
        return
    elseif r then
        self.sync_interval_timer:stop(ev.Loop.default)
        ci.sent_sync_sequence_id = ci.sent_sync_sequence_id + 1
    end
end

function m:listen(time, from_me)
    if from_me then
        self.send[C.PTP_MSG_TYPE_FOLLOW_UP](self, time + self.outbound_latency)
    end
end

function m:uncalibrated(time, from_me)
    if not from_me then
        local ci = self.clock_info
        local hdr = ffi.cast('msg_hdr_t *', ci.ibuf)
        if self:is_from_current_master(hdr) then
            self.sync_receive_time = time
            local correction_field =
                self.timing_iface:i64_to_internal_time(hdr.correction_field)
            if bit.band(hdr.flag_field[0], C.PTP_TWO_STEP_FLAG) ==
                C.PTP_TWO_STEP_FLAG then
                ci.waiting_for_follow = 1
                ci.recv_sync_sequence_id = hdr.sequence_id
                self.last_sync_correction_field = correction_field
                if ci.oo_followup ~= 0 then
                    ci.oo_followup = 0
                    ci.waiting_for_follow = 0
                    local msg = ci.oo_fu_msg[0]
                    local hdr = msg.hdr
                    if ci.recv_sync_sequence_id == hdr.sequence_id then
                        local pot = self.timing_iface:to_internal_time(
                            msg.precise_origin_timestamp)
                        local cf = self.timing_iface:i64_to_internal_time(
                            hdr.correction_field) +
                            self.last_sync_correction_field
                        self:update_offset(pot, cf)
                        self:update_clock()
                    else
                        self:log("warning", string.format(
                            "OO Followup msg seq id mismatch: %d -> %d",
                            ci.recv_sync_sequence_id, hdr.sequence_id))
                    end
                end
            else
                -- FIXME:  Untested one step clock code
                ffi.copy(ci.sync_msg, ci.ibuf, self.sizeof_sync_msg)
                ci.waiting_for_follow = 0
                local msg = ffi.cast('sync_msg_t *', self.ibuf)
                local origin_timestamp = self.timing:to_internal_time(
                    msg.origin_timestamp)
                self:update_offset(origin_timestamp, correction_field)
                self:update_clock()
            end
            return
        end
        return
    end
end

m.handle_state = {}
m.handle_state[C.PTP_STATE_LISTENING] = m.listen
m.handle_state[C.PTP_STATE_PRE_MASTER] = m.handle_state[C.PTP_STATE_LISTENING]
m.handle_state[C.PTP_STATE_MASTER] = m.handle_state[C.PTP_STATE_LISTENING]
m.handle_state[C.PTP_STATE_PASSIVE] = m.handle_state[C.PTP_STATE_LISTENING]
m.handle_state[C.PTP_STATE_UNCALIBRATED] = m.uncalibrated
m.handle_state[C.PTP_STATE_SLAVE] = m.handle_state[C.PTP_STATE_UNCALIBRATED]

function m:handle(time, from_me)
    local state = self.clock_info.state
    local f = m.handle_state[state]
    if f then
        f(self, time, from_me)
    end
end

return m
