require 'strict'
require 'middleclass'
local sqlite3 = require 'lsqlite3'
local cjson = require 'cjson.safe'
local config = require 'system_config'
local network_db = require 'network_db'
local uuid = require 'uuid'
local ts = require 'timestamps'
local socket = require 'socket'
local https = require 'ssl.https'
local log = require 'log'
local ffi = require 'ffi'
local C = ffi.C
local _pn = require 'push-notifications'

ffi.cdef[[
    typedef int pid_t;

    pid_t fork(void);
    int close(int);
]]

local pn = class('push_notification')

--[[
The server id information in the setup database is a JSON object containing

{
}

]]

function pn:initialize()
    -- Set the default send function that does nothing
    self.send = function(self, notification) end


    -- Look up the server information from the setup database.  If it is not
    -- found.  Thrown an error.
    local db = sqlite3.open('/var/lib/armada/setup.db')
    if not db then
        error("Unable to open setup database")
        return
    end
    for r in db:rows(string.format([[
        SELECT content FROM tsd_users WHERE db_name = 'push_notifications' AND
            deleted = '' AND content LIKE '%%"active":true%%';
        ]])) do
        self.server_info = cjson.decode(r[1])
        if not self.server_info then
            error("Unable to decode push notification server record")
            db:close()
            return
        end
        break
    end
    db:close()

    self.server_info = config.fake_push_server_info or self.server_info

    -- Make sure we found a server record
    if not self.server_info or not self.server_info.id then
        -- No record found, no error, notification send function does nothing
        return
    end

    -- Use the real send function
    self.send = self._send
end


function pn:_send(notification)
    -- Make sure server info was found
    if not self.server_info  or not self.server_info.id then
        error("Push notifications no valid server information")
        return
    end

    -- Make sure the argument is the expected type
    if not notification or type(notification) ~= 'table' then
        error("Push notification 'send' function requires 'table' argument")
        return
    end

    -- Make sure the "required" information was given
    if not notification.subject or not notification.body or
        type(notification.subject) ~= 'string' or
        type(notification.body) ~= 'string' then
        error("Sending push notifications requires 'Subject' " ..
            "and 'Body' to be provided")
        return
    end

    log:log(log.INFO, string.format("Sending push notification\n%s",
        cjson.encode(notification)))

    return _pn.send(self.server_info.id, notification.subject,
        notification.body)
end

return pn
